package com.netum.example;

import android.Manifest;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.netum.device.callback.BleNotifyCallback;
import com.netum.device.callback.SppConnectCallback;
import com.netum.device.callback.SppScanAndConnectCallback;
import com.netum.device.data.BleScanState;
import com.netum.example.adapter.DeviceAdapter;
import com.netum.example.common.ObserverManager;
import com.netum.example.message.MessageActivity;
import com.netum.device.SPPManager;
import com.netum.device.callback.BleGattCallback;
import com.netum.device.callback.BleMtuChangedCallback;
import com.netum.device.callback.BleRssiCallback;
import com.netum.device.callback.BleScanCallback;
import com.netum.device.data.BleDevice;
import com.netum.device.exception.BleException;
import com.netum.device.scan.ScanRuleConfig;

import java.util.ArrayList;
import java.util.List;


public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG = MainActivity.class.getSimpleName();
    private static final int REQUEST_CODE_OPEN_GPS = 1;
    private static final int REQUEST_CODE_PERMISSION_LOCATION = 2;

    private LinearLayout layout_setting;
    private TextView txt_setting;
    private Button btn_scan,btn_sdp;
    private EditText et_name, et_mac, et_uuid;
    private Switch sw_auto;
    private ImageView img_loading;

    private Animation operatingAnim;
    private DeviceAdapter mDeviceAdapter;
    private ProgressDialog progressDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        initView();

        SPPManager.getInstance().init(getApplication());
        SPPManager.getInstance()
                .enableLog(true)
                .setReConnectCount(1, 5000)
                .setConnectOverTime(1000)
                .setOperateTimeout(5000);
    }

    @Override
    protected void onResume() {
        super.onResume();
        showConnectedDevice();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        SPPManager.getInstance().disconnectAllDevice();
        SPPManager.getInstance().destroy();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_scan:
                if (btn_scan.getText().equals(getString(R.string.start_scan))) {
                    checkPermissions(true);
                } else if (btn_scan.getText().equals(getString(R.string.stop_scan))) {
                    SPPManager.getInstance().cancelScan();
                }
                break;
            case R.id.btn_sdp:
                if (btn_sdp.getText().equals(getString(R.string.start_sdp))) {
                    checkPermissions(false);
                } else if (btn_sdp.getText().equals(getString(R.string.stop_sdp))) {
                    if(SPPManager.getInstance().getSdpState() == BleScanState.STATE_SCANNING)
                        SPPManager.getInstance().stopSdp();
                }
                break;

            case R.id.txt_setting:
                if (layout_setting.getVisibility() == View.VISIBLE) {
                    layout_setting.setVisibility(View.GONE);
                    txt_setting.setText(getString(R.string.expand_search_settings));
                } else {
                    layout_setting.setVisibility(View.VISIBLE);
                    txt_setting.setText(getString(R.string.retrieve_search_settings));
                }
                break;
        }
    }

    private void initView() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        //toolbar.inflateMenu(R.menu.menu_main);
        //toolbar.setOnMenuItemClickListener((Toolbar.OnMenuItemClickListener) this);


        btn_scan = (Button) findViewById(R.id.btn_scan);
        btn_scan.setText(getString(R.string.start_scan));
        btn_scan.setOnClickListener(this);

        btn_sdp = (Button) findViewById(R.id.btn_sdp);
        btn_sdp.setText(getString(R.string.start_sdp));
        btn_sdp.setOnClickListener(this);

        et_name = (EditText) findViewById(R.id.et_name);
        et_mac = (EditText) findViewById(R.id.et_mac);
        et_uuid = (EditText) findViewById(R.id.et_uuid);
        sw_auto = (Switch) findViewById(R.id.sw_auto);

        layout_setting = (LinearLayout) findViewById(R.id.layout_setting);
        txt_setting = (TextView) findViewById(R.id.txt_setting);
        txt_setting.setOnClickListener(this);
        layout_setting.setVisibility(View.GONE);
        txt_setting.setText(getString(R.string.expand_search_settings));

        img_loading = (ImageView) findViewById(R.id.img_loading);
        operatingAnim = AnimationUtils.loadAnimation(this, R.anim.rotate);
        operatingAnim.setInterpolator(new LinearInterpolator());
        progressDialog = new ProgressDialog(this);

        mDeviceAdapter = new DeviceAdapter(this);
        mDeviceAdapter.setOnDeviceClickListener(new DeviceAdapter.OnDeviceClickListener() {
            @Override
            public void onConnect(BleDevice bleDevice) {
                if (!SPPManager.getInstance().isConnected(bleDevice)) {
                    SPPManager.getInstance().cancelScan();
                    connect(bleDevice);
                }
            }

            @Override
            public void onDisConnect(final BleDevice bleDevice) {
                if (SPPManager.getInstance().isConnected(bleDevice)) {
                    SPPManager.getInstance().disconnect(bleDevice);
                }
            }

            @Override
            public void onDetail(BleDevice bleDevice) {
                if (SPPManager.getInstance().isConnected(bleDevice)) {
                    Toast.makeText(MainActivity.this, "Developing", Toast.LENGTH_LONG).show();
                    //Intent intent = new Intent(MainActivity.this, OperationActivity.class);
                    //intent.putExtra(OperationActivity.KEY_DATA, bleDevice);
                    //startActivity(intent);
                }
            }
        });
        ListView listView_device = (ListView) findViewById(R.id.list_device);
        listView_device.setAdapter(mDeviceAdapter);
    }

    private void showConnectedDevice() {
        List<BleDevice> deviceList = SPPManager.getInstance().getAllConnectedDevice();
        mDeviceAdapter.clearConnectedDevice();
        for (BleDevice bleDevice : deviceList) {
            mDeviceAdapter.addDevice(bleDevice);
        }
        mDeviceAdapter.notifyDataSetChanged();
    }
    /**
     * Start Sdp
     */
    private void startSdp() {
        SPPManager.getInstance().startSdp(
                new SppScanAndConnectCallback() //SDP发现服务回调函数
                {
                    @Override
                    public void onScanStarted(boolean success) {
                        //开启服务（主线程）
                        mDeviceAdapter.clearScanDevice();
                        mDeviceAdapter.notifyDataSetChanged();
                        btn_sdp.setText(getString(R.string.stop_sdp));
                    }

                    @Override
                    public void onLeScan(BleDevice bleDevice) {
                        super.onLeScan(bleDevice);
                    }

                    @Override
                    public void onScanning(BleDevice bleDevice) {
                        //显示一个主动请求的的SPP设备（主线程）
                        mDeviceAdapter.addDevice(bleDevice);
                        mDeviceAdapter.notifyDataSetChanged();
                    }

                    @Override
                    public void onScanFinished(List<BleDevice> scanResultList) {
                        //停止服务，列出所有主动请求的的SPP设备（主线程）
                        btn_sdp.setText(getString(R.string.start_sdp));
                    }

                    @Override
                    public void onStartConnect() {
                        progressDialog.show();
                    }//开始连接

                    @Override
                    public void onConnectFail(BleDevice bleDevice, BleException exception) {
                        //连接失败
                        progressDialog.dismiss();
                        Toast.makeText(MainActivity.this, getString(R.string.connect_fail), Toast.LENGTH_LONG).show();
                    }

                    @Override
                    public void onConnectSuccess(BleDevice bleDevice, BluetoothSocket socket, int status) {
                        //连接成功
                        progressDialog.dismiss();
                        mDeviceAdapter.addDevice(bleDevice);
                        mDeviceAdapter.notifyDataSetChanged();
                    }

                    @Override
                    public void onDataReceiving(final BleDevice bleDevice, byte[] data) {
                        //接收数据
                        final String message = new String(data);
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                if(MessageActivity.Instance!=null)
                                {
                                    MessageActivity.Instance.addMessage(bleDevice.getName()+"-"+bleDevice.getMac(),message);
                                }else {
                                    Toast.makeText(MainActivity.this, message, Toast.LENGTH_LONG).show();
                                }
                                //addText(txt, HexUtil.formatHexString(characteristic.getValue(), true));
                            }
                        });
                    }

                    @Override
                    public void onDisConnected(boolean isActiveDisConnected, BleDevice bleDevice, BluetoothSocket socket, int status) {
                        //断开连接
                        progressDialog.dismiss();

                        mDeviceAdapter.removeDevice(bleDevice);
                        mDeviceAdapter.notifyDataSetChanged();

                        if (isActiveDisConnected) {
                            Toast.makeText(MainActivity.this, getString(R.string.active_disconnected), Toast.LENGTH_LONG).show();
                        } else {
                            Toast.makeText(MainActivity.this, getString(R.string.disconnected), Toast.LENGTH_LONG).show();
                            ObserverManager.getInstance().notifyObserver(bleDevice);
                        }
                    }
                });
    }

    private void setScanRule() {
        String[] names;
        String str_name = et_name.getText().toString();
        if (TextUtils.isEmpty(str_name)) {
            names = null;
        } else {
            names = str_name.split(",");
        }

        String mac = et_mac.getText().toString();

        boolean isAutoConnect = sw_auto.isChecked();
        ScanRuleConfig scanRuleConfig = new ScanRuleConfig.Builder()
                .setDeviceName(true, names)   // 只扫描指定广播名的设备，可选
                .setDeviceMac(mac)                  // 只扫描指定mac的设备，可选
                .setAutoConnect(isAutoConnect)      // 连接时的autoConnect参数，可选，默认false
                .setScanTimeOut(10000)              // 扫描超时时间，可选，默认10秒
                .setFilter(true)                   // 设置是否过滤设备，可选，默认true
                .build();
        SPPManager.getInstance().initScanRule(scanRuleConfig);
    }

    private void startScan() {
        SPPManager.getInstance().scan(new BleScanCallback() {
            @Override
            public void onScanStarted(boolean success) {
                //开始扫描（主线程）
                mDeviceAdapter.clearScanDevice();
                mDeviceAdapter.notifyDataSetChanged();
                img_loading.startAnimation(operatingAnim);
                img_loading.setVisibility(View.VISIBLE);
                btn_scan.setText(getString(R.string.stop_scan));
            }

            @Override
            public void onLeScan(BleDevice bleDevice) {
                super.onLeScan(bleDevice);
            }

            @Override
            public void onScanning(BleDevice bleDevice) {
                //扫描到一个符合扫描规则的SPP设备（主线程）
                mDeviceAdapter.addDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();
            }

            @Override
            public void onScanFinished(List<BleDevice> scanResultList) {
                //扫描结束，列出所有扫描到的符合扫描规则的SPP设备（主线程）
                img_loading.clearAnimation();
                img_loading.setVisibility(View.INVISIBLE);
                btn_scan.setText(getString(R.string.start_scan));
            }
        });
    }

    private void connect(final BleDevice bleDevice) {
        SPPManager.getInstance().connect(bleDevice, new SppConnectCallback() {
            @Override
            public void onStartConnect() {
                progressDialog.show();
            }//开始连接

            @Override
            public void onConnectFail(BleDevice bleDevice, BleException exception) {
                //连接失败
                progressDialog.dismiss();
                Toast.makeText(MainActivity.this, getString(R.string.connect_fail), Toast.LENGTH_LONG).show();
            }

            @Override
            public void onConnectSuccess(BleDevice bleDevice, BluetoothSocket socket, int status) {
                //连接成功
                progressDialog.dismiss();
                mDeviceAdapter.addDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();
            }

            @Override
            public void onDataReceiving(final BleDevice bleDevice, byte[] data) {
                //数据接收
                final String message = new String(data);
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if(MessageActivity.Instance!=null)
                        {
                            MessageActivity.Instance.addMessage(bleDevice.getName()+"-"+bleDevice.getMac(),message);
                        }else {
                            Toast.makeText(MainActivity.this, message, Toast.LENGTH_LONG).show();
                        }
                        //addText(txt, HexUtil.formatHexString(characteristic.getValue(), true));
                    }
                });
            }

            @Override
            public void onDisConnected(boolean isActiveDisConnected, BleDevice bleDevice, BluetoothSocket socket, int status) {
                //断开连接
                progressDialog.dismiss();

                mDeviceAdapter.removeDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();

                if (isActiveDisConnected) {
                    Toast.makeText(MainActivity.this, getString(R.string.active_disconnected), Toast.LENGTH_LONG).show();
                } else {
                    Toast.makeText(MainActivity.this, getString(R.string.disconnected), Toast.LENGTH_LONG).show();
                    ObserverManager.getInstance().notifyObserver(bleDevice);
                }
            }
        });
    }

    @Override
    public final void onRequestPermissionsResult(int requestCode,
                                                 @NonNull String[] permissions,
                                                 @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_CODE_PERMISSION_LOCATION:
                if (grantResults.length > 0) {
                    for (int i = 0; i < grantResults.length; i++) {
                        if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                            onPermissionGranted(permissions[i],true);
                        }
                    }
                }
                break;
        }
    }

    private void checkPermissions(boolean isScan) {
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (!bluetoothAdapter.isEnabled()) {
            Toast.makeText(this, getString(R.string.please_open_blue), Toast.LENGTH_LONG).show();
            return;
        }

        String[] permissions = {Manifest.permission.ACCESS_FINE_LOCATION};
        List<String> permissionDeniedList = new ArrayList<>();
        for (String permission : permissions) {
            int permissionCheck = ContextCompat.checkSelfPermission(this, permission);
            if (permissionCheck == PackageManager.PERMISSION_GRANTED) {
                onPermissionGranted(permission,isScan);
            } else {
                permissionDeniedList.add(permission);
            }
        }
        if (!permissionDeniedList.isEmpty()) {
            String[] deniedPermissions = permissionDeniedList.toArray(new String[permissionDeniedList.size()]);
            ActivityCompat.requestPermissions(this, deniedPermissions, REQUEST_CODE_PERMISSION_LOCATION);
        }
    }

    private void onPermissionGranted(String permission,boolean isScan) {
        switch (permission) {
            case Manifest.permission.ACCESS_FINE_LOCATION:
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && !checkGPSIsOpen()) {
                    new AlertDialog.Builder(this)
                            .setTitle(R.string.notifyTitle)
                            .setMessage(R.string.gpsNotifyMsg)
                            .setNegativeButton(R.string.cancel,
                                    new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            finish();
                                        }
                                    })
                            .setPositiveButton(R.string.setting,
                                    new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                                            startActivityForResult(intent, REQUEST_CODE_OPEN_GPS);
                                        }
                                    })

                            .setCancelable(false)
                            .show();
                } else {
                    if(isScan){
                        setScanRule();
                        startScan();
                    }else {
                        startSdp();
                    }
                }
                break;
        }
    }

    private boolean checkGPSIsOpen() {
        LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
        if (locationManager == null)
            return false;
        return locationManager.isProviderEnabled(android.location.LocationManager.GPS_PROVIDER);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_CODE_OPEN_GPS) {
            if (checkGPSIsOpen()) {
                setScanRule();
                startScan();
            }
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main,menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()){
            case R.id.action_record:
                Intent intent = new Intent(MainActivity.this, MessageActivity.class);
                startActivity(intent);
                break;
        }
        return super.onOptionsItemSelected(item);
    }
}
